#include "MapsCallbacks.h"

#include "Assets/Maps.h"
#include "Assets/Maps_Animations.h"

#include "../Sprites/Assets/Sprites_Tilesets.h"

#include "../Movables/Movable.h"
#include "../Game/Game.h"
#include "../Fade/Fade.h"
#include "../Inputs/InputManager.h"
#include "../../Game/Lever/Lever.h"
#include "../../Game/TurnManager/TurnManager.h"
#include "../../Game/Helpers/TimerManager.h"
#include "../../Game/PatternManager/PatternManager.h"

/*
* Called when map 01 is loaded
*/
void MapsCallbacks_OnMap01Loaded(MapLoadedParameters* _CustomParameters)
{
    // Local variables
    Movable* playerMovable, *leverMovableA, *leverMovableB, *leverMovableC, *leverMovableD;
    Box playerBox = {70, 42, 16, 16};
    Box leverBoxA = {16, 104, 16, 16};
    Box leverBoxB = {48, 40, 8, 16};
    Box leverBoxC = {24, 80, 8, 16};
    Box leverBoxD = {112, 72, 8, 16};
    Lever *leverA, *leverB, *leverC, *levelD;

    SPRITES_8x16;

    // Load the 'sprites' tiles into sprite memory
	Sprite_LoadSpriteTileset(Sprites_Tileset16x16, SPRITES_TILESET16x16_NBTILE);

    // Create player movable
    playerMovable = Movable_CreateMovable(SPRITES_TILESET16x16_PLAYER_LEFT, SPRITES_TILESET16x16_PLAYER_RIGHT, &playerBox, COLLISIONMODE_COLLISION, TAG_PLAYER, LAYER_0, LAYERMASK_0_ONLY, SCROLLRULE_SCROLLREFERENCE, TRUE);

    // Create lever A
    leverMovableA = Movable_CreateMovable(SPRITES_NOTILE, SPRITES_NOTILE, &leverBoxA, COLLISIONMODE_OVERLAP, TAG_PICKABLE_LEVER_A, LAYER_0, LAYERMASK_ALL, SCROLLRULE_CANSCROLL, TRUE);
    
    if (leverMovableA != NULL)
    {
        Lever_InitLever(LEVER_A, leverMovableA);
        leverMovableA->onOverlap = OnLeverOverlap;
    }

    // Create lever B
    leverMovableB = Movable_CreateMovable(SPRITES_NOTILE, SPRITES_NOTILE, &leverBoxB, COLLISIONMODE_OVERLAP, TAG_PICKABLE_LEVER_A, LAYER_0, LAYERMASK_ALL, SCROLLRULE_CANSCROLL, TRUE);
    
    if (leverMovableB != NULL)
    {
        Lever_InitLever(LEVER_B, leverMovableB);
        leverMovableB->onOverlap = OnLeverOverlap;
    }

    // Create lever C
    leverMovableC = Movable_CreateMovable(SPRITES_NOTILE, SPRITES_NOTILE, &leverBoxC, COLLISIONMODE_OVERLAP, TAG_PICKABLE_LEVER_A, LAYER_0, LAYERMASK_ALL, SCROLLRULE_CANSCROLL, TRUE);
    
    if (leverMovableC != NULL)
    {
        Lever_InitLever(LEVER_C, leverMovableC);
        leverMovableC->onOverlap = OnLeverOverlap;
    }

    // Create lever D
    leverMovableD = Movable_CreateMovable(SPRITES_NOTILE, SPRITES_NOTILE, &leverBoxD, COLLISIONMODE_OVERLAP, TAG_PICKABLE_LEVER_A, LAYER_0, LAYERMASK_ALL, SCROLLRULE_CANSCROLL, TRUE);
    
    if (leverMovableD != NULL)
    {
        Lever_InitLever(LEVER_D, leverMovableD);
        leverMovableD->onOverlap = OnLeverOverlap;
    }

    // Bind map update callback
    Game_BindMapUpdateCallback(MapsCallbacks_OnMap01Update);
    
    // Mark game turn as ready
    IsGameTurnReady = TRUE;
    TimerManager_CreateTimer(12);

    // Fade screen
    /*
    wait_vbl_done();
    FADE_WHITETOSTANDARD(30);
    InputManager_SetInputEnabled(TRUE);
    */
}

/*
* Called when map 02 is loaded
*/
void MapsCallbacks_OnMap02Loaded(MapLoadedParameters* _CustomParameters)
{
    /*
    // Local variables
    Movable* playerMovable, *leverMovableA, *leverMovableB, *leverMovableC, *leverMovableD;
    Box playerBox = {60, 60, 16, 16};
    Box leverBoxA = {26, 18, 8, 16};
    Box leverBoxB = {62, 18, 8, 16};
    Box leverBoxC = {92, 18, 8, 16};
    Box leverBoxD = {120, 18, 8, 16};
    Lever *leverA, *leverB, *leverC, *levelD;

    SPRITES_8x16;

    // Load the 'sprites' tiles into sprite memory
	Sprite_LoadSpriteTileset(Sprites_Tileset16x16, SPRITES_TILESET16x16_NBTILE);

    // Create player movable
    playerMovable = Movable_CreateMovable(SPRITES_TILESET16x16_PLAYER_LEFT, SPRITES_TILESET16x16_PLAYER_RIGHT, &playerBox, COLLISIONMODE_COLLISION, TAG_PLAYER, LAYER_0, LAYERMASK_0_ONLY, SCROLLRULE_SCROLLREFERENCE, FALSE);

    // Create lever A
    leverMovableA = Movable_CreateMovable(SPRITES_TILESET16x16_PICKUP_COIN, SPRITES_NOTILE, &leverBoxA, COLLISIONMODE_OVERLAP, TAG_PICKABLE_LEVER_A, LAYER_0, LAYERMASK_ALL, SCROLLRULE_CANSCROLL, TRUE);
    
    if (leverMovableA != NULL)
    {
        Lever_InitLever(LEVER_A, leverMovableA);
        leverMovableA->onOverlap = OnLeverOverlap;
    }

    // Create lever B
    leverMovableB = Movable_CreateMovable(SPRITES_TILESET16x16_PICKUP_COIN, SPRITES_NOTILE, &leverBoxB, COLLISIONMODE_OVERLAP, TAG_PICKABLE_LEVER_A, LAYER_0, LAYERMASK_ALL, SCROLLRULE_CANSCROLL, TRUE);
    
    if (leverMovableB != NULL)
    {
        Lever_InitLever(LEVER_B, leverMovableB);
        leverMovableB->onOverlap = OnLeverOverlap;
    }

    // Create lever C
    leverMovableC = Movable_CreateMovable(SPRITES_TILESET16x16_PICKUP_COIN, SPRITES_NOTILE, &leverBoxC, COLLISIONMODE_OVERLAP, TAG_PICKABLE_LEVER_A, LAYER_0, LAYERMASK_ALL, SCROLLRULE_CANSCROLL, TRUE);
    
    if (leverMovableC != NULL)
    {
        Lever_InitLever(LEVER_C, leverMovableC);
        leverMovableC->onOverlap = OnLeverOverlap;
    }

    // Create lever D
    leverMovableD = Movable_CreateMovable(SPRITES_TILESET16x16_PICKUP_COIN, SPRITES_NOTILE, &leverBoxD, COLLISIONMODE_OVERLAP, TAG_PICKABLE_LEVER_A, LAYER_0, LAYERMASK_ALL, SCROLLRULE_CANSCROLL, TRUE);
    
    if (leverMovableD != NULL)
    {
        Lever_InitLever(LEVER_D, leverMovableD);
        leverMovableD->onOverlap = OnLeverOverlap;
    }

    // Mark game turn as ready
    IsGameTurnReady = TRUE;
    TimerManager_CreateTimer(10);
    */
}

/*
* Called when map 03 is loaded
*/
void MapsCallbacks_OnMap03Loaded(MapLoadedParameters* _CustomParameters)
{
    /*
    // Local variables
    Movable* playerMovable, *leverMovableA, *leverMovableB, *leverMovableC, *leverMovableD;
    Box playerBox = {60, 60, 16, 16};
    Box leverBoxA = {26, 18, 8, 16};
    Box leverBoxB = {62, 18, 8, 16};
    Box leverBoxC = {92, 18, 8, 16};
    Box leverBoxD = {120, 18, 8, 16};
    Lever *leverA, *leverB, *leverC, *levelD;

    SPRITES_8x16;

    // Load the 'sprites' tiles into sprite memory
	Sprite_LoadSpriteTileset(Sprites_Tileset16x16, SPRITES_TILESET16x16_NBTILE);

    // Create player movable
    playerMovable = Movable_CreateMovable(SPRITES_TILESET16x16_PLAYER_LEFT, SPRITES_TILESET16x16_PLAYER_RIGHT, &playerBox, COLLISIONMODE_COLLISION, TAG_PLAYER, LAYER_0, LAYERMASK_0_ONLY, SCROLLRULE_SCROLLREFERENCE, FALSE);

    // Create lever A
    leverMovableA = Movable_CreateMovable(SPRITES_TILESET16x16_PICKUP_COIN, SPRITES_NOTILE, &leverBoxA, COLLISIONMODE_OVERLAP, TAG_PICKABLE_LEVER_A, LAYER_0, LAYERMASK_ALL, SCROLLRULE_CANSCROLL, TRUE);
    
    if (leverMovableA != NULL)
    {
        Lever_InitLever(LEVER_A, leverMovableA);
        leverMovableA->onOverlap = OnLeverOverlap;
    }

    // Create lever B
    leverMovableB = Movable_CreateMovable(SPRITES_TILESET16x16_PICKUP_COIN, SPRITES_NOTILE, &leverBoxB, COLLISIONMODE_OVERLAP, TAG_PICKABLE_LEVER_A, LAYER_0, LAYERMASK_ALL, SCROLLRULE_CANSCROLL, TRUE);
    
    if (leverMovableB != NULL)
    {
        Lever_InitLever(LEVER_B, leverMovableB);
        leverMovableB->onOverlap = OnLeverOverlap;
    }

    // Create lever C
    leverMovableC = Movable_CreateMovable(SPRITES_TILESET16x16_PICKUP_COIN, SPRITES_NOTILE, &leverBoxC, COLLISIONMODE_OVERLAP, TAG_PICKABLE_LEVER_A, LAYER_0, LAYERMASK_ALL, SCROLLRULE_CANSCROLL, TRUE);
    
    if (leverMovableC != NULL)
    {
        Lever_InitLever(LEVER_C, leverMovableC);
        leverMovableC->onOverlap = OnLeverOverlap;
    }

    // Create lever D
    leverMovableD = Movable_CreateMovable(SPRITES_TILESET16x16_PICKUP_COIN, SPRITES_NOTILE, &leverBoxD, COLLISIONMODE_OVERLAP, TAG_PICKABLE_LEVER_A, LAYER_0, LAYERMASK_ALL, SCROLLRULE_CANSCROLL, TRUE);
    
    if (leverMovableD != NULL)
    {
        Lever_InitLever(LEVER_D, leverMovableD);
        leverMovableD->onOverlap = OnLeverOverlap;
    }

    // Mark game turn as ready
    IsGameTurnReady = TRUE;
    TimerManager_CreateTimer(5);
    */
}

/*
* Called when map game over is loaded
*/
void MapsCallbacks_OnMapGameOverLoaded(MapLoadedParameters* _CustomParameters)
{
    HIDE_SPRITES;
    delay(3000);
    MapLoader_RequestLoadMapData(MAP01_LOAD_PARAMS);
    SHOW_SPRITES;
}

/*
* Called at map update. Play background animations for map 01
*/
void MapsCallbacks_OnMap01Update()
{
    /*
    BackgroundAnim_UpdateBackgroundAnim(&BackgroundAnim_WaterFall);
    BackgroundAnim_UpdateBackgroundAnim(&BackgroundAnim_WaterSplash);
    BackgroundAnim_UpdateBackgroundAnim(&BackgroundAnim_WaterIlde0);
    BackgroundAnim_UpdateBackgroundAnim(&BackgroundAnim_WaterIlde1);
    */
}