#include "TurnManager.h"

#include "../../Engine/Dialogs/DialogManager.h"
#include "../PatternManager/PatternManager.h"
#include "../../Engine/Maps/MapLoader.h"
#include "../Helpers/TimerManager.h"

#include <rand.h>

// Current player ID
UINT8 CurrentPlayerID = 0;

// Is turn ready to be played
UINT8 IsGameTurnReady = FALSE;

/** Exposed Functions **/

/*
* Called each frame
*/
void TurnManager_Update()
{
    if (IsGameTurnReady)
    {
        // Update game timer
        TimerManager_UpdateTimer(&GameTimer);

        // Check game timer reached
        if (TimerManager_IsTimerReached(&GameTimer))
        {
            // Game over
            TurnManager_GameOver();
        }
    }
}

/*
* Draw next player turn data as player ID and pattern and load a level randomly
*/
void TurnManager_GoToNextTurn()
{
    IsGameTurnReady = FALSE;

    // Get next player
    CurrentPlayerID = (CurrentPlayerID + 1) % NB_MAX_PLAYERS;

    // Get random pattern
    PatternManager_GeneratePattern();

    // Draw next player and pattern
    TurnManager_DrawNextPlayerAndPattern();

    // Load random map
    TurnManager_RequestRandomLevelLoading();
}

/*
* Go to game over screen
*/
void TurnManager_GameOver()
{
    IsGameTurnReady = FALSE;

    CurrentPattern.levers[0] = LEVER_A;
    CurrentPattern.levers[1] = LEVER_B;
    CurrentPattern.levers[2] = LEVER_C;
    CurrentPattern.levers[3] = LEVER_D;

    MapLoader_RequestLoadMapData(MAP_GAMEOVER_LOAD_PARAMS);
}

/** Static Functions **/

/*
* Draw next player ID and next pattern
*/
static void TurnManager_DrawNextPlayerAndPattern()
{
    // Local variables
    Dialog dialogPattern, dialogPlayer;

    DialogSentence dialogSentencePattern = 
    {
        {
        0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,
        0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,
        0x32+192,0x11+192+CurrentPattern.levers[0],0x32+192,0x32+192,0x32+192,0x32+192,0x11+192+CurrentPattern.levers[1],0x32+192,0x32+192,0x32+192,0x32+192,0x11+192+CurrentPattern.levers[2],0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x11+192+CurrentPattern.levers[3],0x32+192,
        0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,
        0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192,0x32+192
        },
        NULL
    };

    // Open pattern dialog
    dialogPattern.activeSentence = &dialogSentencePattern;
    DialogManager_Open(&dialogPattern, NULL);
    wait_vbl_done();
    delay(600);
    DialogManager_Close();

    // Open player dialog
    dialogPlayer.activeSentence = TurnManager_GetDialogSentenceFromPlayerID();
    DialogManager_Open(&dialogPlayer, NULL);
    wait_vbl_done();
    delay(600);
    DialogManager_Close();
}

/*
* Load a random level (delayed task)
*/
static void TurnManager_RequestRandomLevelLoading()
{
    /*
    // Local variables
    UINT8 randMapID = (UINT8)rand();

    if (randMapID < 80)
    {
        MapLoader_RequestLoadMapData(MAP01_LOAD_PARAMS);
    }
    else if (randMapID < 160)
    {
        MapLoader_RequestLoadMapData(MAP02_LOAD_PARAMS);
    }
    else
    {
        MapLoader_RequestLoadMapData(MAP03_LOAD_PARAMS);
    }
    */

    MapLoader_RequestLoadMapData(MAP01_LOAD_PARAMS);
}

/*
* Get dialog sentence corresponding to current player ID
*/
static DialogSentence* TurnManager_GetDialogSentenceFromPlayerID()
{
    switch (CurrentPlayerID)
    {
        case 0:
            return &DialogSentence_Player0;

        case 1:
            return &DialogSentence_Player1;

        case 2:
            return &DialogSentence_Player2;
    }

    return NULL;
}